From 4640109c9c7a58a7d42931ee7e829c4f98f3b5a7 Mon Sep 17 00:00:00 2001
From: Timo Sirainen <timo.sirainen@open-xchange.com>
Date: Tue, 31 Aug 2021 12:08:32 +0300
Subject: [PATCH 1/3] indexer: Use a separate indexer_queue_callback_t type for
 indexer-queue callback

---
 src/indexer/indexer-queue.c | 4 ++--
 src/indexer/indexer-queue.h | 4 +++-
 2 files changed, 5 insertions(+), 3 deletions(-)

diff --git a/src/indexer/indexer-queue.c b/src/indexer/indexer-queue.c
index bc7718791f..e0239a351a 100644
--- a/src/indexer/indexer-queue.c
+++ b/src/indexer/indexer-queue.c
@@ -7,7 +7,7 @@
 #include "indexer-queue.h"
 
 struct indexer_queue {
-	indexer_status_callback_t *callback;
+	indexer_queue_callback_t *callback;
 	void (*listen_callback)(struct indexer_queue *);
 
 	/* username+mailbox -> indexer_request */
@@ -29,7 +29,7 @@ static int indexer_request_cmp(const struct indexer_request *r1,
 }
 
 struct indexer_queue *
-indexer_queue_init(indexer_status_callback_t *callback)
+indexer_queue_init(indexer_queue_callback_t *callback)
 {
 	struct indexer_queue *queue;
 	
diff --git a/src/indexer/indexer-queue.h b/src/indexer/indexer-queue.h
index fe56829faa..a9d5b9d1e7 100644
--- a/src/indexer/indexer-queue.h
+++ b/src/indexer/indexer-queue.h
@@ -3,6 +3,8 @@
 
 #include "indexer.h"
 
+typedef void indexer_queue_callback_t(int status, void *context);
+
 struct indexer_request {
 	struct indexer_request *prev, *next;
 
@@ -30,7 +32,7 @@ struct indexer_request {
 	ARRAY(void *) contexts;
 };
 
-struct indexer_queue *indexer_queue_init(indexer_status_callback_t *callback);
+struct indexer_queue *indexer_queue_init(indexer_queue_callback_t *callback);
 void indexer_queue_deinit(struct indexer_queue **queue);
 
 /* The callback is called whenever a new request is added to the queue. */
-- 
GitLab


From dd4eb3dc08a8fd3dc5f1afff30e2d7389c5840a8 Mon Sep 17 00:00:00 2001
From: Timo Sirainen <timo.sirainen@open-xchange.com>
Date: Tue, 31 Aug 2021 12:12:49 +0300
Subject: [PATCH 2/3] indexer: Don't free worker_connection too early

There's no need for the status callback anymore to free the connection.
It will be tracked automatically.
---
 src/indexer/indexer.c           |  2 --
 src/indexer/worker-connection.c | 17 +----------------
 src/indexer/worker-connection.h |  2 +-
 src/indexer/worker-pool.c       |  8 +-------
 src/indexer/worker-pool.h       |  2 --
 5 files changed, 3 insertions(+), 28 deletions(-)

diff --git a/src/indexer/indexer.c b/src/indexer/indexer.c
index 508c185512..9a7af6e28c 100644
--- a/src/indexer/indexer.c
+++ b/src/indexer/indexer.c
@@ -108,8 +108,6 @@ static void worker_status_callback(int percentage, void *context)
 
 	indexer_queue_request_finish(queue, &request,
 				     percentage == 100);
-	if (worker_pool != NULL) /* not in deinit */
-		worker_pool_release_connection(worker_pool, conn);
 
 	/* if this was the last request for the connection, we can send more
 	   through it. delay it a bit, since we may be coming here from
diff --git a/src/indexer/worker-connection.c b/src/indexer/worker-connection.c
index 90f536d2b2..a9b9ae464c 100644
--- a/src/indexer/worker-connection.c
+++ b/src/indexer/worker-connection.c
@@ -25,8 +25,6 @@
 struct worker_connection {
 	struct connection conn;
 
-	int refcount;
-
 	indexer_status_callback_t *callback;
 
 	char *request_username;
@@ -44,7 +42,7 @@ static void worker_connection_call_callback(struct worker_connection *worker,
 		worker->request = NULL;
 }
 
-static void worker_connection_destroy(struct connection *conn)
+void worker_connection_destroy(struct connection *conn)
 {
 	struct worker_connection *worker =
 		container_of(conn, struct worker_connection, conn);
@@ -52,18 +50,6 @@ static void worker_connection_destroy(struct connection *conn)
 	worker->request = NULL;
 	i_free_and_null(worker->request_username);
 	connection_deinit(conn);
-}
-
-void worker_connection_unref(struct connection **_conn)
-{
-	struct connection *conn = *_conn;
-	struct worker_connection *worker =
-		container_of(conn, struct worker_connection, conn);
-
-	i_assert(worker->refcount > 0);
-	if (--worker->refcount > 0)
-		return;
-	worker_connection_destroy(conn);
 	i_free(conn);
 }
 
@@ -220,7 +206,6 @@ worker_connection_create(const char *socket_path,
 	struct worker_connection *conn;
 
 	conn = i_new(struct worker_connection, 1);
-	conn->refcount = 1;
 	conn->callback = callback;
 	connection_init_client_unix(list, &conn->conn, socket_path);
 
diff --git a/src/indexer/worker-connection.h b/src/indexer/worker-connection.h
index 58fd050660..f8d6c39d1a 100644
--- a/src/indexer/worker-connection.h
+++ b/src/indexer/worker-connection.h
@@ -10,7 +10,7 @@ struct connection *
 worker_connection_create(const char *socket_path,
 			 indexer_status_callback_t *callback,
 			 struct connection_list *list);
-void worker_connection_unref(struct connection **_conn);
+void worker_connection_destroy(struct connection *conn);
 
 struct connection_list *worker_connection_list_create(void);
 
diff --git a/src/indexer/worker-pool.c b/src/indexer/worker-pool.c
index 6367265885..16f8feb6e3 100644
--- a/src/indexer/worker-pool.c
+++ b/src/indexer/worker-pool.c
@@ -60,7 +60,7 @@ static int worker_pool_add_connection(struct worker_pool *pool,
 	conn = worker_connection_create(pool->socket_path, pool->callback,
 					pool->connection_list);
 	if (connection_client_connect(conn) < 0) {
-		worker_connection_unref(&conn);
+		worker_connection_destroy(conn);
 		return -1;
 	}
 
@@ -99,12 +99,6 @@ bool worker_pool_get_connection(struct worker_pool *pool,
 	return TRUE;
 }
 
-void worker_pool_release_connection(struct worker_pool *pool ATTR_UNUSED,
-				    struct connection *conn)
-{
-	worker_connection_unref(&conn);
-}
-
 struct connection *
 worker_pool_find_username_connection(struct worker_pool *pool,
 				     const char *username)
diff --git a/src/indexer/worker-pool.h b/src/indexer/worker-pool.h
index 057d67f37c..3f114d2024 100644
--- a/src/indexer/worker-pool.h
+++ b/src/indexer/worker-pool.h
@@ -13,8 +13,6 @@ bool worker_pool_have_busy_connections(struct worker_pool *pool);
 
 bool worker_pool_get_connection(struct worker_pool *pool,
 				struct connection **conn_r);
-void worker_pool_release_connection(struct worker_pool *pool,
-				    struct connection *conn);
 
 struct connection *
 worker_pool_find_username_connection(struct worker_pool *pool,
-- 
GitLab


From 6c047bd28caaa4d203a3f2ec43f199c83ec792f3 Mon Sep 17 00:00:00 2001
From: Timo Sirainen <timo.sirainen@open-xchange.com>
Date: Tue, 31 Aug 2021 12:14:15 +0300
Subject: [PATCH 3/3] indexer: Change status callback to take struct
 indexer_request parameter

---
 src/indexer/indexer.c           |  6 ++----
 src/indexer/indexer.h           |  5 ++++-
 src/indexer/worker-connection.c | 10 +---------
 src/indexer/worker-connection.h |  3 ---
 4 files changed, 7 insertions(+), 17 deletions(-)

diff --git a/src/indexer/indexer.c b/src/indexer/indexer.c
index 9a7af6e28c..219af539f4 100644
--- a/src/indexer/indexer.c
+++ b/src/indexer/indexer.c
@@ -95,11 +95,9 @@ static void queue_listen_callback(struct indexer_queue *queue)
 	queue_try_send_more(queue);
 }
 
-static void worker_status_callback(int percentage, void *context)
+static void
+worker_status_callback(int percentage, struct indexer_request *request)
 {
-	struct connection *conn = context;
-	struct indexer_request *request = worker_connection_get_request(conn);
-
 	if (percentage >= 0 && percentage < 100) {
 		indexer_queue_request_status(queue, request,
 					     percentage);
diff --git a/src/indexer/indexer.h b/src/indexer/indexer.h
index 417a433cd9..2234f2db70 100644
--- a/src/indexer/indexer.h
+++ b/src/indexer/indexer.h
@@ -1,8 +1,11 @@
 #ifndef INDEXER_H
 #define INDEXER_H
 
+struct indexer_request;
+
 /* percentage: -1 = failed, 0..99 = indexing in progress, 100 = done */
-typedef void indexer_status_callback_t(int percentage, void *context);
+typedef void
+indexer_status_callback_t(int percentage, struct indexer_request *request);
 
 void indexer_refresh_proctitle(void);
 
diff --git a/src/indexer/worker-connection.c b/src/indexer/worker-connection.c
index a9b9ae464c..d50655f0a3 100644
--- a/src/indexer/worker-connection.c
+++ b/src/indexer/worker-connection.c
@@ -37,7 +37,7 @@ static void worker_connection_call_callback(struct worker_connection *worker,
 					    int percentage)
 {
 	if (worker->request != NULL)
-		worker->callback(percentage, &worker->conn);
+		worker->callback(percentage, worker->request);
 	if (percentage < 0 || percentage == 100)
 		worker->request = NULL;
 }
@@ -168,14 +168,6 @@ const char *worker_connection_get_username(struct connection *conn)
 	return worker->request_username;
 }
 
-struct indexer_request *
-worker_connection_get_request(struct connection *conn)
-{
-	struct worker_connection *worker =
-		container_of(conn, struct worker_connection, conn);
-	return worker->request;
-}
-
 static const struct connection_vfuncs worker_connection_vfuncs = {
 	.destroy = worker_connection_destroy,
 	.input_args = worker_connection_input_args,
diff --git a/src/indexer/worker-connection.h b/src/indexer/worker-connection.h
index f8d6c39d1a..c307a63fd5 100644
--- a/src/indexer/worker-connection.h
+++ b/src/indexer/worker-connection.h
@@ -35,7 +35,4 @@ bool worker_connection_is_busy(struct connection *conn);
    or NULL if there are none. */
 const char *worker_connection_get_username(struct connection *conn);
 
-struct indexer_request *
-worker_connection_get_request(struct connection *conn);
-
 #endif
-- 
GitLab

